<?php
/**
 * Allows to add comments to a wiki page
 * The discussion is stored in a separate file ns:discussion:wikipage
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author  Esther Brunner <esther@kaffeehaus.ch>
 * @author  Dave Lawson <dlawson@masterytech.com>
 * @author  Pavel Vitis <pavel [dot] vitis [at] seznam [dot] cz>
 */

global $ID;
global $ACT;
global $conf;

// use if-modified-since header?
if (blog_getConf('useifmodifiedsince')
  && !$_REQUEST['first']
  && !headers_sent()
  && ($ACT == 'show')
  && $INFO['exists'])
  blog_imsHeader();

if ($_REQUEST['blog'] == 'feed')
  require_once(DOKU_PLUGIN.'blog/inc/feed.php');

// user wants to create new blog entry
if ($_POST['blog'] == 'newentry') blog_newEntry();

// load MTC if enabled
if (blog_getConf('comment_engine') == 'mtc'){
  require_once(DOKU_PLUGIN.'blog/inc/mtc.class.php');
  $MTC = new MTC();
  $MTC->db_host   = blog_getConf('mtc_db_host');
  $MTC->db_user   = blog_getConf('mtc_db_user');
  $MTC->db_pass   = blog_getConf('mtc_db_pass');
  $MTC->db_name   = blog_getConf('mtc_db_name');
  $MTC->addcss    = true; // set this to false if you want to use a custom style defined in style.css
  if ($conf['usewordblock']) $MTC->blacklist = DOKU_CONF.'wordblock.conf';
  if ($conf['notify']) $MTC->notify = $conf['notify'];
  $MTC->adminpass = blog_getConf('mtc_adminpass');
  $MTC->page      = $ID;
  $MTC->init();
}
  
// check if user wants to add a comment
if (blog_isDiscussionID($ID) && file_exists(wikiFN($ID))){
  session_write_close();

  // Comment was submitted, let's save it
  if ($_POST['blog'] == 'addcomment'){
    $fullname = $_REQUEST['fullname'];
    $email    = $_REQUEST['email'];
    $comment  = $_REQUEST['wikitext'];
    // Save only if comment is not empty
    if (trim($comment, " \t\n") != ''){
      require_once(DOKU_PLUGIN.'blog/inc/comment.php');
      blog_saveComment($ID, $fullname, $email, $comment);
    }
  }

  // don't show discussion files, show main files instead
  $dNS = cleanID(blog_getConf('discussion'));
  if ($dNS && $ACT == 'show' && !headers_sent()){
    header("Location: ".wl(blog_stripDiscussionNS($ID), '', true).'#'.$dNS);
    exit();
  }
}

/**
 * Main template function to display discussion text. Call this in your template
 * to display the comment form and existing comments.
 *
 * @author Dave Lawson <dlawson@masterytech.com>
 */
function html_discussion(){
  tpl_discussion();
}

/**
 * @author  Esther Brunner <esther@kaffeehaus.ch>
 * @author  Dave Lawson <dlawson@masterytech.com>
 */
function tpl_discussion(){
  global $ID;
  global $ACT;
  global $lang;

  $dID = blog_addDiscussionNS($ID);
  $dNS = cleanID(blog_getConf('discussion'));
  $dFN = wikiFN($dID);
          
  // check mode and existing page
  if (!$dNS || $ACT !== 'show' || !file_exists($dFN)) return '';
  
  echo '<div class="discussion">';
  echo '<h1><a name="'.$dNS.'">'.blog_getConf('discussion_title').'</a></h1>';
  echo '<div class="level1">';
  
  if (blog_getConf('comment_engine') == 'mtc'){
    $MTC->comments();
    $MTC->comment_form();
  } else {
    require_once(DOKU_PLUGIN.'blog/inc/comment.php');
    if (blog_getConf('comment_recentfirst')){
      echo blog_commentForm($dID);
      echo blog_discussionPage($dID);
    } else {
      echo blog_discussionPage($dID);
      echo blog_commentForm($dID);
    }
  }
  echo '</div>';
  echo '</div>';
}

/**
 * Is specified page ID in a discussion namespace?
 * 
 * @author Dave Lawson <dlawson@masterytech.com>
 */
function blog_isDiscussionID($id){
  $dNS = cleanID(blog_getConf('discussion'));
  if (!$dNS) return false;
  
  $pID = noNS($id);
  return strpos($id, $dNS.':'.$pID) !== false;
}

/**
 * Convert a discussion page ID to a normal page ID.
 * 
 * @author Dave Lawson <dlawson@masterytech.com>
 */
function blog_stripDiscussionNS($id){
  $dNS = cleanID(blog_getConf('discussion'));
  if (!$dNS) return $id;
  return str_replace($dNS.':', '', $id);
}

/**
 * Convert a normal page ID to a discussion page ID.
 * 
 * @author Dave Lawson <dlawson@masterytech.com>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_addDiscussionNS($id){
  if (blog_isDiscussionID($id)) return $id;
  
  $dNS = cleanID(blog_getConf('discussion'));
  if (!$dNS) return false;
  $dID = getNS($id);
  $dID .= ($dID ? ':' : '').$dNS.':'.noNS($id);
  return $dID;
}

/**
 * Create a new blog entry
 *
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_newEntry(){
  global $lang;
  global $ACT;
  global $ID;
  global $TEXT;
  
  $title = $_REQUEST['title'];
  $ns    = $_REQUEST['ns'];
  $id    = $ns.':'.cleanID(str_replace(':', '', $title));
  
  if (auth_quickaclcheck($id) >= AUTH_CREATE){
    //check if locked by anyone - if not lock for my self
    $ID   = $id;
    $file = wikiFN($ID);
    if(checklock($ID)){
      $ACT  = 'locked';
    } elseif (@file_exists($file)){
      $ACT  = 'edit';
    } else {
      lock($ID);
      $TEXT = pageTemplate($ns.':'.$title);
      $ACT  = 'preview'; // we need preview because edit doesn't allow passing text
    }
  } else {
    $ACT = 'show';
  }
}

/**
 * Send If-Modified-Since Header
 * We use only a pseudo time for Last-Modified and refresh after an hour
 *
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_imsHeader(){
  global $conf;
  
  // shared caching (proxy, for example) is forbidden if user is logged in
  if (auth_aclcheck(getNS($MEDIA.':X'),'','') < AUTH_READ){
    header('Cache-Control: private, must-revalidate');
  } else {
    header('Cache-Control: public, must-revalidate');
  }
  
  // has a If-Modified-Since header been sent?
  if ($_SERVER['HTTP_IF_MODIFIED_SINCE']){
    $cmod = explode(';', $_SERVER['HTTP_IF_MODIFIED_SINCE']);
    $cmod = strtotime($cmod[0]);
  } else {
    $cmod = 0;
  }
  
  // check if-modified-since date and send 304 status message if nothing has
  // changed since last time
  if ($cmod && ($cmod >= @filemtime($conf['changelog']))){
    header('HTTP/1.1 304 Not Modified');
    exit();
  } else {
    header('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT');
  }
}

/**
 * use this function to access plugin configuration variables
 */
function blog_getConf($id){
  global $blog_configloaded;
  global $conf;

  if (!$blog_configloaded){
    if ($pconf = blog_loadConfig()){
      foreach ($pconf as $key => $value){
        if (isset($conf['plugin']['blog'][$key])) continue;
        $conf['plugin']['blog'][$key] = $value;
      }
      $blog_configloaded = true;
    }
  }
  
  return $conf['plugin']['blog'][$id];
}

/**
 * reads all plugin configuration variables into $this->conf
 * this function is automatically called by getConf()
 */
function blog_loadConfig(){
  
  $path = DOKU_PLUGIN.'blog/conf/';
  $conf = array();
  
  if (!@file_exists($path.'default.php')) return false;
  
  // load default config file
  require_once($path.'default.php');
    
  return $conf;
}
 
//Setup VIM: ex: et ts=2 enc=utf-8 :
