<?php

/**
 * Get a section including its subsections
 */
function incl_getSection($title, $instructions){
  foreach ($instructions as $instruction){
    if ($instruction[0] == 'header'){
  
      // found the right header
      if (cleanID($instruction[1][0]) == $title){
        $level = $instruction[1][1];
        $i[] = $instruction;
      
      // next header of the same or higher level -> exit
      } elseif ($instruction[1][1] <= $level){
        return $i;
      } elseif (isset($level)){
        $i[] = $instruction;
      }
      
    // add instructions from our section
    } elseif (isset($level)){
      $i[] = $instruction;
    }
  }
  return $i;
}

/**
 * Corrects relative internal links and media and
 * converts headers of included pages to subheaders of the current page
 */
function incl_convertInstructions($instr, $incl, &$renderer, $clevel, &$ok, $firstseconly=false){
  global $ID;
  global $conf;
  
  // check if included page is in same namespace
  $iNS = getNS($incl);
  if (getNS($ID) == $iNS) $convert = false;
  else $convert = true;
    
  $n = count($instr);
  for ($i = 0; $i < $n; $i++){
  
    // convert internal links and media from relative to absolute
    if ($convert && (substr($instr[$i][0], 0, 8) == 'internal')){
      
      // relative subnamespace
      if ($instr[$i][1][0]{0} == '.'){
        $instr[$i][1][0] = $iNS.':'.substr($instr[$i][1][0], 1);
          
      // relative link
      } elseif (strpos($instr[$i][1][0],':') === false) {
        $instr[$i][1][0] = $iNS.':'.$instr[$i][1][0];
      }
    
    // set header level to current section level + header level
    } elseif ($instr[$i][0] == 'header'){
      $level = $instr[$i][1][1]+$clevel;
      if ($level > 5) $level = 5;
      $instr[$i][1][1] = $level;
      
      // add TOC items
      if ($level >= $conf['toptoclevel'] && $level <= $conf['maxtoclevel']){
        $text = $instr[$i][1][0];
        $hid  = $renderer->_headerToLink($text, 'true');
        $renderer->toc[] = array(
          'hid'   => $hid,
          'title' => $text,
          'type'  => 'ul',
          'level' => $level-$conf['toptoclevel']+1
        );
        
        $ok = true;
      }
      
    // the same for sections
    } elseif ($instr[$i][0] == 'section_open'){
      $level = $instr[$i][1][0]+$clevel;
      if ($level > 5) $level = 5;
      $instr[$i][1][0] = $level;
      
    // show only the first section?
    } elseif ($firstseconly && ($instr[$i][0] == 'section_close') && ($instr[$i-1][0] != 'section_open')){
      if ($instr[0][0] == 'document_start') return array_slice($instr, 1, $i);
      else return array_slice($instr, 0, $i);
      
    }
  }
  if ($instr[0][0] == 'document_start') return array_slice($instr, 1, -1);
  else return $instr;
}

/**
 * Remove TOC, section edit buttons and tags
 */
function incl_cleanXHTML($xhtml, &$tags){
  preg_match('!<div class="blog_tags">.*?</div>!s', $xhtml, $match);
  $tags    = $match[0];
  $replace = array(
    '!<div class="toc">.*?</div>\n</div>!s' => '', // remove toc
    '#<!-- SECTION \[\d*-\d*\] -->#e'       => '', // remove section edit buttons
    '!<div class="blog_tags">.*?</div>!s'   => ''  // remove category tags
  );
  return preg_replace(array_keys($replace), array_values($replace), $xhtml);
}

/**
 * Display an edit button for the included page
 */
function incl_editButton($id, $file, $perm){
  $ret = '';
  if (@file_exists($file)){
    if (($perm >= AUTH_EDIT) && (is_writable($file))) $ret = '<div class="secedit">'.html_btn('edit', $id, '', array('do' => 'edit'), 'post').'</div>';
  } elseif ($perm >= AUTH_CREATE){
    $ret = '<div class="secedit">'.html_btn('create', $id, '', array('do' => 'edit'), 'post').'</div>';
  }
  return $ret;
}

//Setup VIM: ex: et ts=2 enc=utf-8 :
