<?php
/**
 * Blog Plugin: displays a number of recent entries from the blog subnamespace
 * 
 * @license  GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author   Esther Brunner <esther [at] kaffeehaus [dot] ch>
 * @author   Robert Rackl <wiki [at] doogie [dot] de>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_blog extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-04-14',
      'name'   => 'Blog Plugin (blog component)',
      'desc'   => 'Displays a number of recent entries from a given namesspace',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog',
    );
  }

  function getType(){ return 'substition'; }
  function getPType(){ return 'block'; }
  function getSort(){ return 307; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern('\{\{blog>.+?\}\}',$mode,'plugin_blog_blog'); }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 7, -2); // strip {{blog> from start and }} from end
    list($ns, $num) = explode('?', $match, 2);
    if (!is_numeric($num)){
      if (is_numeric($ns)){
        $num = $ns;
        $ns  = '';
      } else {
        $num = 5;
      }
    }

    return array($ns, $num);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data){
    require_once(DOKU_PLUGIN.'blog/inc/recent.php');
    require_once(DOKU_PLUGIN.'blog/inc/include.php');
    require_once(DOKU_PLUGIN.'blog/functions.php');
    global $conf;
    global $ID;
    global $filechain;
    global $auth;
    
    $ns    = $data[0];
    if ($ns == '') $ns = cleanID(blog_getConf('blog_namespace'));
    
    $num   = $data[1];
    $first = $_REQUEST['first'];
    if (!is_numeric($first)) $first = 0;
    
    // get recent pages from our blog namespace
    $flags  = RECENTS_SKIP_DELETED + RECENTS_SKIP_MINORS + RECENTS_SKIP_SUBSPACES;
    $recents = blog_recents($first, $num+1, $ns, $flags);
    if (count($recents) == 0 && $first != 0)
      $recents = blog_recents(0, $num+1, $ns, $flags);
    if (count($recents) > $num){
      unset($recents[$num]);
      $more = 1;
    } else {
      $num  = count($recents);
      $more = 0;
    }
              
    if ($mode == 'xhtml'){
      if (!isset($filechain)) $filechain[] = $ID;
      define('IS_BLOG_MAINPAGE', 1);
            
      // prevent caching to ensure the included pages are always fresh
      $renderer->info['cache'] = false;

      // current section level
      preg_match_all('|<div class="level(\d)">|i', $renderer->doc, $matches, PREG_SET_ORDER);
      $n = count($matches)-1;
      if ($n > -1) $clevel = $matches[$n][1];
      else $clevel = 0;
      
      // close current section
      if ($clevel) $renderer->doc .= '</div>';
            
      foreach ($recents as $recent){
        $id   = $recent['id'];
        $file = wikiFN($id);
        
        // don't show pages already included
        if (in_array($id, $filechain)) continue;
        $filechain[] = $id;
                        
        // get blog entry instructions and render them on the fly
        $firstseconly = blog_getConf('blog_firstseconly');
        $instr   = p_cached_instructions(wikiFN($id));
        $instr   = incl_convertInstructions($instr, $id, $renderer, $clevel, $ok, $firstseconly);
        if ($firstseconly) $instr = $this->_readMore($instr, $id);
        $content = p_render('xhtml', $instr, $info);
        $content = incl_cleanXHTML($content, $tags);
                
        // output the content
        $renderer->doc .= '<div class="include">';
        if (!$ok && $clevel) $renderer->doc .= '<div class="level'.$clevel.'">';
        $renderer->doc .= $content.incl_editButton($id, $file, $perm);
        if (!$ok && $clevel) $renderer->doc .= '</div>';
        $renderer->doc .= '</div>';
        $renderer->doc .= $this->_byline($id, $recent, $renderer, $tags);
      }
      
      // resume the section
      if ($clevel) $renderer->doc .= '<div class="level'.$clevel.'">';
            
      // show older / newer entries links
      $renderer->doc .= $this->_browseEntriesLinks($more, $first, $data[1], $ID);
      
      // show new entry form
      if (auth_quickaclcheck($ns.':*') >= AUTH_CREATE)
        $renderer->doc .= $this->_newEntryForm($ns);
                  
      return true;
      
    // for metadata renderer
    } elseif ($mode == 'metadata'){
      foreach ($recents as $recent){
        $id = $recent['id'];
        $renderer->meta['relation']['haspart'][$id] = true;
        // $meta = array('relation' => array('ispartof' => array($ID => true)));
        // p_set_metadata($id, $meta);
      }
      
      return true;
    }
    return false;
  }
  
  /**
   * adds a read more... link at the bottom of the first section
   */
  function _readMore($instr, $id){
    $last    = $instr[count($instr)-1];
    if ($last[0] == 'section_close') $instr = array_slice($instr, 0, -1);
    $instr[] = array('p_open', array(), $last[2]);
    $instr[] = array('internallink', array($id, $this->getLang('readmore')), $last[2]);
    $instr[] = array('p_close', array(), $last[2]);
    if ($last[0] == 'section_close') $instr[] = $last;
    return $instr;
  }
  
  /**
   * returns the byline below the blog entry
   */
  function _byline($id, $recent, &$renderer, $tags=''){
    global $conf;
    global $auth;
    
    $ret = $tags;
    
    // author info
    if (blog_getConf('blog_showuser') && $recent['user'] && !is_null($auth)){
      $userInfo = $auth->getUserData($recent['user']);
      $author   = $userInfo['name'];
      $ret .= $renderer->internallink(cleanID(blog_getConf('tag_namespace').':'.$author), $author, '', true).' &middot; ';
    }
    
    // date permalink
    if (blog_getConf('blog_showdate')){
      $date = date($conf['dformat'], $recent['date']);
      $ret .= $renderer->internallink($id, $date, '', true);
    }
        
    // MTC comments
    if (blog_getConf('comment_engine') == 'mtc'){
      $ret .= ' &middot; '.$renderer->internallink($id.'#'.cleanID(blog_getConf('discussion')), $this->getLang('comments'), '', true);
       
    // wiki discussion page
    } else {
      $dFN = wikiFN(blog_addDiscussionNS($id));
      if (@file_exists($dFN)){
        $text = io_readFile($dFN);
        $noc  = count(explode('----',$text))-1;
        $ret .= ' &middot; '.$renderer->internallink($id.'#'.cleanID(blog_getConf('discussion')), $this->getLang('comments').' ('.$noc.')', '', true);
      }
    }
    
    if (!$ret) $ret = '&nbsp;';
    return '<div class="blogmeta">'.$ret.'</div>';
  }
  
  /**
   * displays links to older newer entries of the blog namespace
   */
  function _browseEntriesLinks($more, $first, $num, $id){
    $ret = '';
    $last = $first+$num;
    if ($first > 0){
      $first -= $num;
      if ($first < 0) $first = 0;
      $ret .= '<p class="centeralign"><a href="'.wl($id,'first='.$first).'" class="wikilink1">&lt;&lt; '.$this->getLang('newer').'</a>';
      if ($more) $ret .= ' | ';
      else $ret .= '</p>';
    } else if ($more){
      $ret .= '<p class="centeralign">';
    }
    if ($more){
      $ret .= '<a href="'.wl($id,'first='.$last).'" class="wikilink1">'.$this->getLang('older').' &gt;&gt;</a></p>';
    }
    return $ret;
  }
  
  /**
   * displays a form to enter the title of a new entry in the blog namespace
   * and then open that page in the edit mode
   */
  function _newEntryForm($ns){
    global $lang;
    
    $ret = '<hr /><div class="centeralign"><form id="blog__newentry_form" method="post" action="'.script().'" accept-charset="'.$lang['encoding'].'"><div class="no">'.
      '<input type="hidden" name="ns" value="'.$ns.'" />'.
      '<input type="hidden" name="do" value="edit" />'.
      '<input type="hidden" name="blog" value="newentry" />'.
      '<label class="block" for="blog__newentry_title"><span>'.$this->getLang('newentry').'</span> <input class="edit" type="text" name="title" id="blog__newentry_title" size="40" tabindex="1" /></label>'.
      '<input class="button" type="submit" value="'.$lang['btn_create'].'" tabindex="2" />'.
      '</div></form></div>';
    return $ret;
  }
      
}

//Setup VIM: ex: et ts=4 enc=utf-8 :
