<?php
/**
 * Editor Plugin: displays links to all wiki pages edited by a given user or ip
 * 
 * @license  GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author   Esther Brunner <esther@kaffeehaus.ch>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_PLUGIN.'blog/functions.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_editor extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-04-14',
      'name'   => 'Blog Plugin (editor component)',
      'desc'   => 'Displays a list of wiki pages from a given month or with a given category tag',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog',
    );
  }

  function getType(){ return 'substition'; }
  function getPType(){ return 'block'; }
  function getSort(){ return 309; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern('\{\{editor>.+?\}\}',$mode,'plugin_blog_editor'); }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 9, -2); // strip {{editor> from start and }} from end
    list($ns, $rest) = explode("?", $match);
    if (!$rest){
      $rest = $ns;
      $ns   = '';
    }
    
    if (preg_match("/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/", $rest)) $type = 'ip';
    else $type = 'user';    
    
    return array($type, $ns, $rest);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
    global $ID;
    global $conf;
    global $auth;
    
    $ns   = $data[1];
    if ($ns == '') $ns = cleanID(blog_getConf('blog_namespace'));
    $user = $data[2];
    
    $pages = $this->_editorArchive($ns, $user, $data[0], $more);
    
    if (!count($pages)) return true; // nothing to display
    
    if ($mode == 'xhtml'){

      // prevent caching to ensure content is always fresh
      $renderer->info['cache'] = false;
        
      $renderer->doc .= '<table class="archive">';
      foreach ($pages as $page){
        $renderer->doc .= '<tr><td class="page">';
        $id  = $page['id'];
        $title = p_get_first_heading($id);
        $renderer->doc .= $renderer->internallink(':'.$id, $title).'</td>';
        if (blog_getConf('archive_showdate')){
          $renderer->doc .= '<td class="date">'.date($conf['dformat'],$page['date']).'</td>';
        }
        if (($user == '@ALL') && $page['user'] && blog_getConf('archive_showuser') && !is_null($auth)){
          $userInfo = $auth->getUserData($page['user']);
          $renderer->doc .= '<td class="user">'.$userInfo['name'].'</td>';
        }
        $renderer->doc .= '</tr>';
      }
      $renderer->doc .= '</table>';
      
      return true;
      
    // for metadata renderer
    } elseif ($mode == 'metadata'){
      foreach ($pages as $page){
        $id  = $page['id'];
        $renderer->meta['relation']['references'][$id] = true;
        // $meta = array('relation' => array('isreferencedby' => array($ID => true)));
        // p_set_metadata($id, $meta);
      }
      
      return true;
    }
    return false;
  }
  
  /**
   * return the monthly archive list
   */
  function _editorArchive($ns, $user, $type, &$more){
    global $conf;
    require_once(DOKU_PLUGIN.'blog/inc/recent.php');
    
    $result = array();
    $num    = $conf['recent'];
    $flags  = RECENTS_SKIP_DELETED + RECENTS_SKIP_MINORS;
    if ($ns == '*') $ns = '';
    else $flags += RECENTS_SKIP_SUBSPACES;
    $first  = $_REQUEST['first'];
    if (!is_numeric($first)) $first = 0;
        
    $result = blog_recents(0, $num+1, $ns, $flags, $type, $user);
        
    if (count($result) == 0 && $first != 0)
      $result = blog_recents(0, $num+1, $ns, $flags, $type, $user);
    if (count($result) > $num){
      unset($result[$num]);
      $more = 1;
    } else {
      $more = 0;
    }
            
    return $result;
  }
        
}

//Setup VIM: ex: et ts=4 enc=utf-8 :