<?php
/**
 * Feed Plugin: creates a feed link for a given blog namespace
 * 
 * @license  GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author   Esther Brunner <esther [at] kaffeehaus [dot] ch>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_PLUGIN.'blog/functions.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_feed extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-04-14',
      'name'   => 'Blog Plugin (feed component)',
      'desc'   => 'Displays a number of recent entries from a given namesspace',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog:new',
    );
  }

  function getType(){ return 'substition'; }
  function getSort(){ return 308; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern('\{\{feed>.+?\}\}',$mode,'plugin_blog_feed'); }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 7, -2); // strip {{feed> from start and }} from end
    list($match, $title) = explode('|', $match, 2);
    list($ns, $num) = explode('?', $match, 2);
    if (!is_numeric($num)) $num = '';
    if ($ns{0} == '@'){
      $type = 'tag';
      $ns   = substr($ns, 1);
    } else {
      $type = 'ns';
    }

    return array($type, $ns, $num, $title);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data){
    $ns    = $data[1];
    if ($data[0] == 'ns' && !$ns) $ns = blog_getConf('blog_namespace');
    $title = ($data[3] ? $data[3] : $ns);
  
    if($mode == 'xhtml'){
      $url   = DOKU_BASE.'doku.php?blog=feed';
      if ($ns) $url .= '&'.$data[0].'='.cleanID($ns);
      if ($data[2]) $url .= '&num='.$data[2];
      $title = $renderer->_xmlEntities($title);
      
      $renderer->doc .= '<a href="'.$url.'" class="feed" rel="nofollow"'.
        ' type="application/rss+xml" title="'.$title.'">'.$title.'</a>';
                
      return true;
    
    // for metadata renderer
    } elseif ($mode == 'metadata'){
      if ($renderer->capture) $renderer->doc .= $title;
      
      return true;
    }
    return false;
  }
        
}

//Setup VIM: ex: et ts=4 enc=utf-8 :
