<?php
/**
 * Gravatar Plugin: displays gravatar images with syntax{{gravatar>email@domain.com}}
 * Optionally you can add a title attribute: {{gravatar>email@domain.com|My Name}}
 *
 * @license  GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author   Esther Brunner <esther [at] kaffeehaus [dot] ch>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_PLUGIN.'blog/functions.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_gravatar extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-02-27',
      'name'   => 'Blog Plugin (gravatar component)',
      'desc'   => 'Displays Gravatar images',
      'url'    => 'http://wiki.splitbrain.org/plugin:gravatar',
    );
  }

  function getType(){ return 'substition'; }
  function getSort(){ return 315; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern("{{gravatar>.+?}}",$mode,'plugin_blog_gravatar'); }
  
  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 11, -2); // Strip markup
    $match = preg_split('/\|/u', $match, 2); // Split title from URL
    
    // Check alignment
    $ralign = (bool)preg_match('/^ /', $match[0]);
    $lalign = (bool)preg_match('/ $/', $match[0]);
    if ($lalign & $ralign) $align = 'center';
    else if ($ralign)      $align = 'right';
    else if ($lalign)      $align = 'left';
    else                   $align = NULL;
    
    //split into src and size parameter (using the very last questionmark)
    $pos = strrpos($match[0], '?');
    if($pos !== false){
      $src   = substr($match[0], 0, $pos);
      $param = substr($match[0], $pos+1);
      if (preg_match('/^s/', $param))      $size = '20';
      else if (preg_match('/^l/', $param)) $size = '80';
      else if (preg_match('/^m/', $param)) $size = '40';
      else $size = blog_getConf('gravatar_size');
    } else {
      $src   = $match[0];
      $size  = blog_getConf('gravatar_size');
    }
    
    if (!isset($match[1])) $match[1] = NULL;
    return array(trim($src), trim($match[1]), $align, $size);
  } 
 
  /**
   * Create output
   */
  function render($mode, &$renderer, $data){
    // a string to be added to the gravatar url
    // see http://gravatar.com/implement.php#section_1_1
    if (!defined('GRAVATAR_OPTS')) define('GRAVATAR_OPTS','&amp;rating='.blog_getConf('gravatar_rating'));
  
    if($mode == 'xhtml'){
      $default = DOKU_URL.'lib/plugins/blog/images/default.gif';
      $size    = $data[3];
      $email   = $data[0];
      // Do not pass invalid or empty emails to gravatar site...
      if (isvalidemail($email)){
        $src = ml('http://www.gravatar.com/avatar.php?'.
          'gravatar_id='.md5($email).
          '&default='.urlencode($default).
          '&size='.$size.
          GRAVATAR_OPTS, '');
      // Show only default image if invalid or empty email given
      } else {
        $src = $default;
      }
      $title = ($data[1] ? htmlspecialchars($data[1]) : obfuscate($email));
      
      $renderer->doc .= '<img src="'.$src.'" class="media'.$data[2].'"'.
        ' title="'.$title.'" alt="'.$title.'"'.
        ' width="'.$size.'" height="'.$size.'" />';
      return true;
    }
    return false;
  }
     
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :